@@ -3,12 +3,10 @@ package com.fasterxml.jackson.databind;
 import java.text.DateFormat;
 import java.util.*;
 
-import com.fasterxml.jackson.annotation.JsonAutoDetect;
-import com.fasterxml.jackson.annotation.JsonFormat;
-import com.fasterxml.jackson.annotation.JsonInclude;
-import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
-import com.fasterxml.jackson.annotation.PropertyAccessor;
+import com.fasterxml.jackson.annotation.*;
+
 import com.fasterxml.jackson.core.*;
+
 import com.fasterxml.jackson.databind.cfg.*;
 import com.fasterxml.jackson.databind.deser.DeserializationProblemHandler;
 import com.fasterxml.jackson.databind.introspect.*;
@@ -104,9 +102,9 @@ public final class DeserializationConfig
      */
     public DeserializationConfig(BaseSettings base,
             SubtypeResolver str, SimpleMixInResolver mixins,
-            RootNameLookup rootNames)
+            RootNameLookup rootNames, ConfigOverrides configOverrides)
     {
-        super(base, str, mixins, rootNames);
+        super(base, str, mixins, rootNames, configOverrides);
         _deserFeatures = collectFeatureDefaults(DeserializationFeature.class);
         _nodeFactory = JsonNodeFactory.instance;
         _problemHandlers = null;
@@ -116,6 +114,15 @@ public final class DeserializationConfig
         _formatReadFeaturesToChange = 0;
     }
 
+    /**
+     * @deprecated Since 2.8, remove from 2.9 or later
+     */
+    @Deprecated
+    public DeserializationConfig(BaseSettings base, SubtypeResolver str,
+            SimpleMixInResolver mixins, RootNameLookup rootNames) {
+        this(base, str, mixins, rootNames, null);
+    }
+    
     private DeserializationConfig(DeserializationConfig src,
             int mapperFeatures, int deserFeatures,
             int parserFeatures, int parserFeatureMask,
@@ -233,15 +240,14 @@ public final class DeserializationConfig
     }
 
     /**
-     * Copy-constructor used for making a copy to be used by new {@link ObjectMapper}
-     * or {@link ObjectReader}.
+     * Copy-constructor used for making a copy used by new {@link ObjectMapper}.
      *
-     * @since 2.6
+     * @since 2.8
      */
     protected DeserializationConfig(DeserializationConfig src, SimpleMixInResolver mixins,
-            RootNameLookup rootNames)
+            RootNameLookup rootNames, ConfigOverrides configOverrides)
     {
-        super(src, mixins, rootNames);
+        super(src, mixins, rootNames, configOverrides);
         _deserFeatures = src._deserFeatures;
         _problemHandlers = src._problemHandlers;
         _nodeFactory = src._nodeFactory;
@@ -767,29 +773,12 @@ public final class DeserializationConfig
         return getClassIntrospector().forDirectClassAnnotations(this, type, this);
     }
 
-    @Override
-    public VisibilityChecker<?> getDefaultVisibilityChecker()
-    {
-        VisibilityChecker<?> vchecker = super.getDefaultVisibilityChecker();
-        if (!isEnabled(MapperFeature.AUTO_DETECT_SETTERS)) {
-            vchecker = vchecker.withSetterVisibility(Visibility.NONE);
-        }
-        if (!isEnabled(MapperFeature.AUTO_DETECT_CREATORS)) {
-            vchecker = vchecker.withCreatorVisibility(Visibility.NONE);
-        }
-        if (!isEnabled(MapperFeature.AUTO_DETECT_FIELDS)) {
-            vchecker = vchecker.withFieldVisibility(Visibility.NONE);
-        }
-        return vchecker;
-    }
-
     /*
     /**********************************************************
     /* Configuration: default settings with per-type overrides
     /**********************************************************
      */
-    
-    // property inclusion not used on deserialization yet (2.7): may be added in future
+
     @Override
     public JsonInclude.Value getDefaultPropertyInclusion() {
         return EMPTY_INCLUDE;
@@ -797,13 +786,28 @@ public final class DeserializationConfig
 
     @Override
     public JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType) {
+        ConfigOverride overrides = findConfigOverride(baseType);
+        if (overrides != null) {
+            JsonInclude.Value v = overrides.getInclude();
+            if (v != null) {
+                return v;
+            }
+        }
         return EMPTY_INCLUDE;
     }
 
     @Override
-    public JsonFormat.Value getDefaultPropertyFormat(Class<?> baseType) {
-        // !!! TODO: per-type defaults
-        return EMPTY_FORMAT;
+    public JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType,
+            JsonInclude.Value defaultIncl)
+    {
+        ConfigOverride overrides = findConfigOverride(baseType);
+        if (overrides != null) {
+            JsonInclude.Value v = overrides.getInclude();
+            if (v != null) {
+                return v;
+            }
+        }
+        return defaultIncl;
     }
 
     /*
