@@ -7,11 +7,13 @@ import java.util.Locale;
 import java.util.TimeZone;
 
 import com.fasterxml.jackson.annotation.JsonFormat;
+import com.fasterxml.jackson.annotation.JsonInclude;
 import com.fasterxml.jackson.annotation.ObjectIdGenerator;
 import com.fasterxml.jackson.core.JsonGenerator;
 import com.fasterxml.jackson.databind.cfg.ContextAttributes;
 import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
 import com.fasterxml.jackson.databind.introspect.Annotated;
+import com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.ser.*;
 import com.fasterxml.jackson.databind.ser.impl.FailingSerializer;
@@ -153,7 +155,7 @@ public abstract class SerializerProvider
     /* State, for non-blueprint instances: generic
     /**********************************************************
      */
-    
+
     /**
      * For fast lookups, we will have a local non-shared read-only
      * map that contains serializers previously fetched.
@@ -312,7 +314,8 @@ public abstract class SerializerProvider
         
     /*
     /**********************************************************
-    /* DatabindContext implementation
+    /* DatabindContext implementation (and closely related
+    /* but ser-specific)
     /**********************************************************
      */
 
@@ -355,7 +358,14 @@ public abstract class SerializerProvider
     public final JsonFormat.Value getDefaultPropertyFormat(Class<?> baseType) {
         return _config.getDefaultPropertyFormat(baseType);
     }
-    
+
+    /**
+     * @since 2.8
+     */
+    public final JsonInclude.Value getDefaultPropertyInclusion(Class<?> baseType) {
+        return _config.getDefaultPropertyInclusion();
+    }
+
     /**
      * Method for accessing default Locale to use: convenience method for
      *<pre>
@@ -1071,12 +1081,12 @@ public abstract class SerializerProvider
         }
     }
 
-    public final void defaultSerializeNull(JsonGenerator jgen) throws IOException
+    public final void defaultSerializeNull(JsonGenerator gen) throws IOException
     {
         if (_stdNullValueSerializer) { // minor perf optimization
-            jgen.writeNull();
+            gen.writeNull();
         } else {
-            _nullValueSerializer.serialize(null, jgen, this);
+            _nullValueSerializer.serialize(null, gen, this);
         }
     }
 
@@ -1087,13 +1097,95 @@ public abstract class SerializerProvider
      */
 
     /**
+     * Factory method for constructing a {@link JsonMappingException};
+     * usually only indirectly used by calling
+     * {@link #reportMappingProblem(String, Object...)}.
+     *
      * @since 2.6
      */
     public JsonMappingException mappingException(String message, Object... args) {
         if (args != null && args.length > 0) {
             message = String.format(message, args);
         }
-        return JsonMappingException.from(this, message);
+        return JsonMappingException.from(getGenerator(), message);
+    }
+
+    /**
+     * Factory method for constructing a {@link JsonMappingException};
+     * usually only indirectly used by calling
+     * {@link #reportMappingProblem(Throwable, String, Object...)}
+     * 
+     * @since 2.8
+     */
+    protected JsonMappingException mappingException(Throwable t, String message, Object... args) {
+        if (args != null && args.length > 0) {
+            message = String.format(message, args);
+        }
+        return JsonMappingException.from(getGenerator(), message, t);
+    }
+
+    /**
+     * Helper method called to indicate problem; default behavior is to construct and
+     * throw a {@link JsonMappingException}, but in future may collect more than one
+     * and only throw after certain number, or at the end of serialization.
+     *
+     * @since 2.8
+     */
+    public void reportMappingProblem(String message, Object... args) throws JsonMappingException {
+        throw mappingException(message, args);
+    }
+
+    /**
+     * Helper method called to indicate problem; default behavior is to construct and
+     * throw a {@link JsonMappingException}, but in future may collect more than one
+     * and only throw after certain number, or at the end of serialization.
+     *
+     * @since 2.8
+     */
+    public void reportMappingProblem(Throwable t, String message, Object... args) throws JsonMappingException {
+        throw mappingException(t, message, args);
+    }
+
+    /**
+     * Helper method called to indicate problem in POJO (serialization) definitions or settings
+     * regarding specific Java type, unrelated to actual JSON content to map.
+     * Default behavior is to construct and throw a {@link JsonMappingException}.
+     *
+     * @since 2.9
+     */
+    public <T> T reportBadTypeDefinition(BeanDescription bean,
+            String message, Object... args) throws JsonMappingException {
+        if (args != null && args.length > 0) {
+            message = String.format(message, args);
+        }
+        String beanDesc = (bean == null) ? "N/A" : _desc(bean.getType().getGenericSignature());
+        throw mappingException("Invalid type definition for type %s: %s",
+                beanDesc, message);
+    }
+
+    /**
+     * Helper method called to indicate problem in POJO (serialization) definitions or settings
+     * regarding specific property (of a type), unrelated to actual JSON content to map.
+     * Default behavior is to construct and throw a {@link JsonMappingException}.
+     *
+     * @since 2.9
+     */
+    public <T> T reportBadPropertyDefinition(BeanDescription bean, BeanPropertyDefinition prop,
+            String message, Object... args) throws JsonMappingException {
+        if (args != null && args.length > 0) {
+            message = String.format(message, args);
+        }
+        String propName = (prop == null)  ? "N/A" : _quotedString(prop.getName());
+        String beanDesc = (bean == null) ? "N/A" : _desc(bean.getType().getGenericSignature());
+        throw mappingException("Invalid definition for property %s (of type %s): %s",
+                propName, beanDesc, message);
+    }
+
+    /**
+     * @since 2.8
+     */
+    public JsonGenerator getGenerator() {
+        return null;
     }
 
     /*
@@ -1112,11 +1204,10 @@ public abstract class SerializerProvider
                 return;
             }
         }
-        throw JsonMappingException.from(this,
-                "Incompatible types: declared root type ("+rootType+") vs "
-                +value.getClass().getName());
+        reportMappingProblem("Incompatible types: declared root type (%s) vs %s",
+                rootType, value.getClass().getName());
     }
-    
+
     /**
      * Method that will try to find a serializer, either from cache
      * or by constructing one; but will not return an "unknown" serializer
@@ -1169,7 +1260,8 @@ public abstract class SerializerProvider
             /* We better only expose checked exceptions, since those
              * are what caller is expected to handle
              */
-            throw JsonMappingException.from(this, iae.getMessage(), iae);
+            reportMappingProblem(iae, iae.getMessage());
+            return null; // never gets here
         }
 
         if (ser != null) {
@@ -1189,7 +1281,8 @@ public abstract class SerializerProvider
             /* We better only expose checked exceptions, since those
              * are what caller is expected to handle
              */
-            throw JsonMappingException.from(this, iae.getMessage(), iae);
+            reportMappingProblem(iae, iae.getMessage());
+            return null; // never gets here
         }
     
         if (ser != null) {
@@ -1247,6 +1340,20 @@ public abstract class SerializerProvider
     /**********************************************************
      */
 
+    protected String _desc(Object value) {
+        if (value == null) {
+            return "N/A";
+        }
+        return "'"+value+"'";
+    }
+
+    protected String _quotedString(Object value) {
+        if (value == null) {
+            return "N/A";
+        }
+        return String.valueOf(value);
+    }
+
     protected final DateFormat _dateFormat()
     {
         if (_dateFormat != null) {
