@@ -6,6 +6,7 @@ import java.lang.reflect.InvocationTargetException;
 import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonFormat;
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.annotation.ObjectIdGenerator;
 import com.fasterxml.jackson.annotation.ObjectIdGenerators;
@@ -139,7 +140,7 @@ public abstract class BeanDeserializerBase
      * track of recognized but ignorable properties: these will
      * be skipped without errors or warnings.
      */
-    final protected HashSet<String> _ignorableProps;
+    final protected Set<String> _ignorableProps;
 
     /**
      * Flag that can be set to ignore and skip unknown properties.
@@ -206,7 +207,7 @@ public abstract class BeanDeserializerBase
     protected BeanDeserializerBase(BeanDeserializerBuilder builder,
             BeanDescription beanDesc,
             BeanPropertyMap properties, Map<String, SettableBeanProperty> backRefs,
-            HashSet<String> ignorableProps, boolean ignoreAllUnknown,
+            Set<String> ignorableProps, boolean ignoreAllUnknown,
             boolean hasViews)
     {
         super(beanDesc.getType());
@@ -228,6 +229,7 @@ public abstract class BeanDeserializerBase
         _objectIdReader = builder.getObjectIdReader();
         _nonStandardCreation = (_unwrappedPropertyHandler != null)
             || _valueInstantiator.canCreateUsingDelegate()
+            || _valueInstantiator.canCreateUsingArrayDelegate() // new in 2.7
             || _valueInstantiator.canCreateFromObjectWith()
             || !_valueInstantiator.canCreateUsingDefault()
             ;
@@ -354,7 +356,7 @@ public abstract class BeanDeserializerBase
         }
     }
 
-    public BeanDeserializerBase(BeanDeserializerBase src, HashSet<String> ignorableProps)
+    public BeanDeserializerBase(BeanDeserializerBase src, Set<String> ignorableProps)
     {
         super(src._beanType);
         
@@ -378,7 +380,40 @@ public abstract class BeanDeserializerBase
 
         _vanillaProcessing = src._vanillaProcessing;
         _objectIdReader = src._objectIdReader;
-        _beanProperties = src._beanProperties;
+
+        // 01-May-2016, tatu: [databind#1217]: Remove properties from mapping,
+        //    to avoid them being deserialized
+        _beanProperties = src._beanProperties.withoutProperties(ignorableProps);
+    }
+
+    /**
+     * @since 2.8
+     */
+    protected BeanDeserializerBase(BeanDeserializerBase src, BeanPropertyMap beanProps)
+    {
+        super(src._beanType);
+        
+        _classAnnotations = src._classAnnotations;
+        _beanType = src._beanType;
+        
+        _valueInstantiator = src._valueInstantiator;
+        _delegateDeserializer = src._delegateDeserializer;
+        _propertyBasedCreator = src._propertyBasedCreator;
+        
+        _beanProperties = beanProps;
+        _backRefs = src._backRefs;
+        _ignorableProps = src._ignorableProps;
+        _ignoreAllUnknown = src._ignoreAllUnknown;
+        _anySetter = src._anySetter;
+        _injectables = src._injectables;
+        _objectIdReader = src._objectIdReader;
+        
+        _nonStandardCreation = src._nonStandardCreation;
+        _unwrappedPropertyHandler = src._unwrappedPropertyHandler;
+        _needViewProcesing = src._needViewProcesing;
+        _serializationShape = src._serializationShape;
+
+        _vanillaProcessing = src._vanillaProcessing;
     }
     
     @Override
@@ -386,8 +421,19 @@ public abstract class BeanDeserializerBase
 
     public abstract BeanDeserializerBase withObjectIdReader(ObjectIdReader oir);
 
-    public abstract BeanDeserializerBase withIgnorableProperties(HashSet<String> ignorableProps);
+    public abstract BeanDeserializerBase withIgnorableProperties(Set<String> ignorableProps);
 
+    /**
+     * Mutant factory method that custom sub-classes must override; not left as
+     * abstract to prevent more drastic backwards compatibility problems.
+     *
+     * @since 2.8
+     */
+    public BeanDeserializerBase withBeanProperties(BeanPropertyMap props) {
+        throw new UnsupportedOperationException("Class "+getClass().getName()
+                +" does not override `withBeanProperties()`, needs to");
+    }
+    
     /**
      * Fluent factory for creating a variant that can handle
      * POJO output as a JSON Array. Implementations may ignore this request
@@ -415,8 +461,16 @@ public abstract class BeanDeserializerBase
         ExternalTypeHandler.Builder extTypes = null;
         // if ValueInstantiator can use "creator" approach, need to resolve it here...
         SettableBeanProperty[] creatorProps;
+
         if (_valueInstantiator.canCreateFromObjectWith()) {
             creatorProps = _valueInstantiator.getFromObjectArguments(ctxt.getConfig());
+
+            // 21-Jun-2015, tatu: This resolution was duplicated later on and seems like
+            //    it really should be only done at a later point. So commented out in 2.8.
+            //   However, just in case there was a reason for it, leaving commented out
+            //   here instead of immediately removing.
+
+            /*
             // also: need to try to resolve 'external' type ids...
             for (SettableBeanProperty prop : creatorProps) {
                 if (prop.hasValueTypeDeserializer()) {
@@ -429,6 +483,7 @@ public abstract class BeanDeserializerBase
                     }
                 }
             }
+            */
         } else {
             creatorProps = null;
         }
@@ -461,7 +516,7 @@ public abstract class BeanDeserializerBase
             // Need to link managed references with matching back references
             prop = _resolveManagedReferenceProperty(ctxt, prop);
 
-            // [databind#351[: need to wrap properties that require object id resolution.
+            // [databind#351]: need to wrap properties that require object id resolution.
             if (!(prop instanceof ManagedReferenceProperty)) {
                 prop = _resolvedObjectIdProperty(ctxt, prop);
             }
@@ -518,13 +573,11 @@ public abstract class BeanDeserializerBase
                 }
             }
         }
-
         // "any setter" may also need to be resolved now
         if (_anySetter != null && !_anySetter.hasValueDeserializer()) {
             _anySetter = _anySetter.withValueDeserializer(findDeserializer(ctxt,
                     _anySetter.getType(), _anySetter.getProperty()));
         }
-
         // as well as delegate-based constructor:
         if (_valueInstantiator.canCreateUsingDelegate()) {
             JavaType delegateType = _valueInstantiator.getDelegateType(ctxt.getConfig());
@@ -555,7 +608,9 @@ public abstract class BeanDeserializerBase
         }
 
         if (extTypes != null) {
-            _externalTypeIdHandler = extTypes.build();
+            // 21-Jun-2016, tatu: related to [databind#999], may need to link type ids too,
+            //    so need to pass collected properties
+            _externalTypeIdHandler = extTypes.build(_beanProperties);
             // we consider this non-standard, to offline handling
             _nonStandardCreation = true;
         }
@@ -669,22 +724,39 @@ public abstract class BeanDeserializerBase
         }
         // And possibly add more properties to ignore
         if (accessor != null) {
-            String[] ignorals = intr.findPropertiesToIgnore(accessor, false);
-            if (ignorals != null && ignorals.length != 0) {
-                HashSet<String> newIgnored = ArrayBuilders.setAndArray(contextual._ignorableProps, ignorals);
-                contextual = contextual.withIgnorableProperties(newIgnored);
+            JsonIgnoreProperties.Value ignorals = intr.findPropertyIgnorals(accessor);
+            if (ignorals != null) {
+                Set<String> ignored = ignorals.findIgnoredForDeserialization();
+                if (!ignored.isEmpty()) {
+                    Set<String> prev = contextual._ignorableProps;
+                    if ((prev != null) && !prev.isEmpty()) {
+                        ignored = new HashSet<String>(ignored);
+                        ignored.addAll(prev);
+                    }
+                    contextual = contextual.withIgnorableProperties(ignored);
+                }
             }
         }
 
         // One more thing: are we asked to serialize POJO as array?
+        JsonFormat.Value format = findFormatOverrides(ctxt, property, handledType());
         JsonFormat.Shape shape = null;
-        if (accessor != null) {
-            JsonFormat.Value format = intr.findFormat((Annotated) accessor);
-
-            if (format != null) {
+        if (format != null) {
+            if (format.hasShape()) {
                 shape = format.getShape();
             }
+            // 16-May-2016, tatu: How about per-property case-insensitivity?
+            Boolean B = format.getFeature(JsonFormat.Feature.ACCEPT_CASE_INSENSITIVE_PROPERTIES);
+            if (B != null) {
+                // !!! TODO
+                BeanPropertyMap propsOrig = _beanProperties;
+                BeanPropertyMap props = propsOrig.withCaseInsensitivity(B.booleanValue());
+                if (props != propsOrig) {
+                    contextual = contextual.withBeanProperties(props);
+                }
+            }
         }
+
         if (shape == null) {
             shape = _serializationShape;
         }
@@ -729,7 +801,7 @@ public abstract class BeanDeserializerBase
      * in case where object id resolution is required.
      */
     protected SettableBeanProperty _resolvedObjectIdProperty(DeserializationContext ctxt,
-            SettableBeanProperty prop)
+            SettableBeanProperty prop) throws JsonMappingException
     {
         ObjectIdInfo objectIdInfo = prop.getObjectIdInfo();
         JsonDeserializer<Object> valueDeser = prop.getValueDeserializer();
@@ -1073,7 +1145,9 @@ public abstract class BeanDeserializerBase
         } else {
             // should we worry about UUIDs? They should be fine, right?
             // 07-Aug-2014, tatu: Maybe, but not necessarily; had issues with
-            //   Smile format; [Smile#19], possibly related.
+            //   Smile format; [dataformat-smile#19], possibly related.
+            // 01-Sep-2016, tatu: For non-JSON, might want to consider `writeEmbeddedObject`
+            //   but that won't work for default impl (JSON and most dataformats)
             buf.writeObject(rawId);
         }
         JsonParser bufParser = buf.asParser();
@@ -1115,20 +1189,21 @@ public abstract class BeanDeserializerBase
     protected Object deserializeFromObjectUsingNonDefault(JsonParser p,
             DeserializationContext ctxt) throws IOException
     {
-        if (_delegateDeserializer != null) {
+        final JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
+        if (delegateDeser != null) {
             return _valueInstantiator.createUsingDelegate(ctxt,
-                    _delegateDeserializer.deserialize(p, ctxt));
+                    delegateDeser.deserialize(p, ctxt));
         }
         if (_propertyBasedCreator != null) {
             return _deserializeUsingPropertyBased(p, ctxt);
         }
         // should only occur for abstract types...
         if (_beanType.isAbstract()) {
-            throw JsonMappingException.from(p, "Can not instantiate abstract type "+_beanType
-                    +" (need to add/enable type information?)");
+            return ctxt.handleMissingInstantiator(handledType(), p,
+                    "abstract type (need to add/enable type information?)");
         }
-        throw JsonMappingException.from(p, "No suitable constructor found for type "
-                +_beanType+": can not instantiate from JSON object (missing default constructor or creator, or perhaps need to add/enable type information?)");
+        return ctxt.handleMissingInstantiator(_beanType.getRawClass(), p,
+                "no suitable constructor found, can not deserialize from Object value (missing default constructor or creator, or perhaps need to add/enable type information?)");
     }
 
     protected abstract Object _deserializeUsingPropertyBased(final JsonParser p,
@@ -1136,18 +1211,20 @@ public abstract class BeanDeserializerBase
         throws IOException, JsonProcessingException;
 
     @SuppressWarnings("incomplete-switch")
-    public Object deserializeFromNumber(JsonParser p, DeserializationContext ctxt) throws IOException
+    public Object deserializeFromNumber(JsonParser p, DeserializationContext ctxt)
+        throws IOException
     {
         // First things first: id Object Id is used, most likely that's it
         if (_objectIdReader != null) {
             return deserializeFromObjectId(p, ctxt);
         }
-
+        final JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
         switch (p.getNumberType()) {
         case INT:
-            if (_delegateDeserializer != null) {
+            if (delegateDeser != null) {
                 if (!_valueInstantiator.canCreateFromInt()) {
-                    Object bean = _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+                    Object bean = _valueInstantiator.createUsingDelegate(ctxt,
+                            delegateDeser.deserialize(p, ctxt));
                     if (_injectables != null) {
                         injectValues(ctxt, bean);
                     }
@@ -1156,9 +1233,10 @@ public abstract class BeanDeserializerBase
             }
             return _valueInstantiator.createFromInt(ctxt, p.getIntValue());
         case LONG:
-            if (_delegateDeserializer != null) {
+            if (delegateDeser != null) {
                 if (!_valueInstantiator.canCreateFromInt()) {
-                    Object bean = _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+                    Object bean = _valueInstantiator.createUsingDelegate(ctxt,
+                            delegateDeser.deserialize(p, ctxt));
                     if (_injectables != null) {
                         injectValues(ctxt, bean);
                     }
@@ -1168,14 +1246,17 @@ public abstract class BeanDeserializerBase
             return _valueInstantiator.createFromLong(ctxt, p.getLongValue());
         }
         // actually, could also be BigInteger, so:
-        if (_delegateDeserializer != null) {
-            Object bean = _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+        if (delegateDeser != null) {
+            Object bean = _valueInstantiator.createUsingDelegate(ctxt,
+                    delegateDeser.deserialize(p, ctxt));
             if (_injectables != null) {
                 injectValues(ctxt, bean);
             }
             return bean;
         }
-        throw ctxt.instantiationException(handledType(), "no suitable creator method found to deserialize from JSON integer number");
+        return ctxt.handleMissingInstantiator(handledType(), p,
+                "no suitable creator method found to deserialize from Number value (%s)",
+                p.getNumberValue());
     }
 
     public Object deserializeFromString(JsonParser p, DeserializationContext ctxt) throws IOException
@@ -1184,13 +1265,14 @@ public abstract class BeanDeserializerBase
         if (_objectIdReader != null) {
             return deserializeFromObjectId(p, ctxt);
         }
-        
         /* Bit complicated if we have delegating creator; may need to use it,
          * or might not...
          */
-        if (_delegateDeserializer != null) {
+        JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
+        if (delegateDeser != null) {
             if (!_valueInstantiator.canCreateFromString()) {
-                Object bean = _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+                Object bean = _valueInstantiator.createUsingDelegate(ctxt,
+                        delegateDeser.deserialize(p, ctxt));
                 if (_injectables != null) {
                     injectValues(ctxt, bean);
                 }
@@ -1204,15 +1286,16 @@ public abstract class BeanDeserializerBase
      * Method called to deserialize POJO value from a JSON floating-point
      * number.
      */
-    @SuppressWarnings("incomplete-switch")
     public Object deserializeFromDouble(JsonParser p, DeserializationContext ctxt) throws IOException
     {
         NumberType t = p.getNumberType();
         // no separate methods for taking float...
         if ((t == NumberType.DOUBLE) || (t == NumberType.FLOAT)) {
-            if (_delegateDeserializer != null) {
+            JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
+            if (delegateDeser != null) {
                 if (!_valueInstantiator.canCreateFromDouble()) {
-                    Object bean = _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+                    Object bean = _valueInstantiator.createUsingDelegate(ctxt,
+                            delegateDeser.deserialize(p, ctxt));
                     if (_injectables != null) {
                         injectValues(ctxt, bean);
                     }
@@ -1222,10 +1305,14 @@ public abstract class BeanDeserializerBase
             return _valueInstantiator.createFromDouble(ctxt, p.getDoubleValue());
         }
         // actually, could also be BigDecimal, so:
-        if (_delegateDeserializer != null) {
-            return _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+        JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
+        if (delegateDeser != null) {
+            return _valueInstantiator.createUsingDelegate(ctxt,
+                    delegateDeser.deserialize(p, ctxt));
         }
-        throw ctxt.instantiationException(handledType(), "no suitable creator method found to deserialize from JSON floating-point number");
+        return ctxt.handleMissingInstantiator(handledType(), p,
+                "no suitable creator method found to deserialize from Number value (%s)",
+                p.getNumberValue());
     }
 
     /**
@@ -1233,9 +1320,11 @@ public abstract class BeanDeserializerBase
      */
     public Object deserializeFromBoolean(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        if (_delegateDeserializer != null) {
+        JsonDeserializer<Object> delegateDeser = _delegateDeserializer();
+        if (delegateDeser != null) {
             if (!_valueInstantiator.canCreateFromBoolean()) {
-                Object bean = _valueInstantiator.createUsingDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
+                Object bean = _valueInstantiator.createUsingDelegate(ctxt,
+                        delegateDeser.deserialize(p, ctxt));
                 if (_injectables != null) {
                     injectValues(ctxt, bean);
                 }
@@ -1248,28 +1337,16 @@ public abstract class BeanDeserializerBase
 
     public Object deserializeFromArray(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        if (_arrayDelegateDeserializer != null) {
-            try {
-                Object bean = _valueInstantiator.createUsingArrayDelegate(ctxt, _arrayDelegateDeserializer.deserialize(p, ctxt));
-                if (_injectables != null) {
-                    injectValues(ctxt, bean);
-                }
-                return bean;
-            } catch (Exception e) {
-                wrapInstantiationProblem(e, ctxt);
-            }
-        }
+        // note: can not call `_delegateDeserializer()` since order reversed here:
+        JsonDeserializer<Object> delegateDeser = _arrayDelegateDeserializer;
         // fallback to non-array delegate
-        if (_delegateDeserializer != null) {
-            try {
-                Object bean = _valueInstantiator.createUsingArrayDelegate(ctxt, _delegateDeserializer.deserialize(p, ctxt));
-                if (_injectables != null) {
-                    injectValues(ctxt, bean);
-                }
-                return bean;
-            } catch (Exception e) {
-                wrapInstantiationProblem(e, ctxt);
+        if ((delegateDeser != null) || ((delegateDeser = _delegateDeserializer) != null)) {
+            Object bean = _valueInstantiator.createUsingArrayDelegate(ctxt,
+                    delegateDeser.deserialize(p, ctxt));
+            if (_injectables != null) {
+                injectValues(ctxt, bean);
             }
+            return bean;
         }
         if (ctxt.isEnabled(DeserializationFeature.UNWRAP_SINGLE_VALUE_ARRAYS)) {
             JsonToken t = p.nextToken();
@@ -1278,8 +1355,7 @@ public abstract class BeanDeserializerBase
             }
             final Object value = deserialize(p, ctxt);
             if (p.nextToken() != JsonToken.END_ARRAY) {
-                throw ctxt.wrongTokenException(p, JsonToken.END_ARRAY, 
-                        "Attempted to unwrap single value array for single '" + _valueClass.getName() + "' value but there was more than a single value in the array");
+                handleMissingEndArrayForSingle(p, ctxt);
             }
             return value;
         }
@@ -1288,9 +1364,10 @@ public abstract class BeanDeserializerBase
             if (t == JsonToken.END_ARRAY) {
                 return null;
             }
-            throw ctxt.mappingException(handledType(), JsonToken.START_ARRAY);
+            return ctxt.handleUnexpectedToken(handledType(),
+                    JsonToken.START_ARRAY, p, null);
         }
-        throw ctxt.mappingException(handledType());
+        return ctxt.handleUnexpectedToken(handledType(), p);
     }
 
     public Object deserializeFromEmbedded(JsonParser p, DeserializationContext ctxt)
@@ -1307,6 +1384,17 @@ public abstract class BeanDeserializerBase
         return p.getEmbeddedObject();
     }
 
+    /**
+     * @since 2.9
+     */
+    private final JsonDeserializer<Object> _delegateDeserializer() {
+        JsonDeserializer<Object> deser = _delegateDeserializer;
+        if (deser == null) {
+            deser = _arrayDelegateDeserializer;
+        }
+        return deser;
+    }
+
     /*
     /**********************************************************
     /* Overridable helper methods
@@ -1545,7 +1633,7 @@ public abstract class BeanDeserializerBase
         return t;
     }
 
-    protected void wrapInstantiationProblem(Throwable t, DeserializationContext ctxt)
+    protected Object wrapInstantiationProblem(Throwable t, DeserializationContext ctxt)
         throws IOException
     {
         while (t instanceof InvocationTargetException && t.getCause() != null) {
@@ -1564,6 +1652,6 @@ public abstract class BeanDeserializerBase
                 throw (RuntimeException) t;
             }
         }
-        throw ctxt.instantiationException(_beanType.getRawClass(), t);
+        return ctxt.handleInstantiationProblem(_beanType.getRawClass(), null, t);
     }
 }
