@@ -2,17 +2,16 @@ package com.fasterxml.jackson.databind.deser;
 
 import java.util.*;
 
-import com.fasterxml.jackson.annotation.ObjectIdGenerator;
-import com.fasterxml.jackson.annotation.ObjectIdGenerators;
-import com.fasterxml.jackson.annotation.ObjectIdResolver;
+import com.fasterxml.jackson.annotation.*;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
 import com.fasterxml.jackson.databind.cfg.DeserializerFactoryConfig;
+import com.fasterxml.jackson.databind.cfg.ConfigOverride;
 import com.fasterxml.jackson.databind.deser.impl.*;
 import com.fasterxml.jackson.databind.deser.std.ThrowableDeserializer;
 import com.fasterxml.jackson.databind.introspect.*;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
-import com.fasterxml.jackson.databind.util.ArrayBuilders;
 import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.SimpleBeanPropertyDefinition;
 
@@ -116,7 +115,7 @@ public class BeanDeserializerFactory
          */
         // 29-Nov-2015, tatu: Also, filter out calls to primitive types, they are
         //    not something we could materialize anything for
-        if (type.isAbstract() && !type.isPrimitive()) {
+        if (type.isAbstract() && !type.isPrimitive() && !type.isEnumType()) {
             // Let's make it possible to materialize abstract types.
             JavaType concreteType = materializeAbstractType(ctxt, type, beanDesc);
             if (concreteType != null) {
@@ -127,7 +126,6 @@ public class BeanDeserializerFactory
                 return buildBeanDeserializer(ctxt, concreteType, beanDesc);
             }
         }
-
         // Otherwise, may want to check handlers for standard types, from superclass:
         @SuppressWarnings("unchecked")
         JsonDeserializer<Object> deser = (JsonDeserializer<Object>) findStdDeserializer(ctxt, type, beanDesc);
@@ -220,7 +218,7 @@ public class BeanDeserializerFactory
         try {
             valueInstantiator = findValueInstantiator(ctxt, beanDesc);
         } catch (NoClassDefFoundError error) {
-            return new NoClassDefFoundDeserializer<Object>(error);
+            return new ErrorThrowingDeserializer(error);
         }
         BeanDeserializerBuilder builder = constructBeanDeserializerBuilder(ctxt, beanDesc);
         builder.setValueInstantiator(valueInstantiator);
@@ -233,7 +231,6 @@ public class BeanDeserializerFactory
         addInjectables(ctxt, beanDesc, builder);
         
         final DeserializationConfig config = ctxt.getConfig();
-        // [JACKSON-440]: update builder now that all information is in?
         if (_factoryConfig.hasDeserializerModifiers()) {
             for (BeanDeserializerModifier mod : _factoryConfig.deserializerModifiers()) {
                 builder = mod.updateBuilder(config, beanDesc, builder);
@@ -448,29 +445,42 @@ public class BeanDeserializerFactory
                 builder.getValueInstantiator().getFromObjectArguments(ctxt.getConfig());
         final boolean isConcrete = !beanDesc.getType().isAbstract();
         
-        // Things specified as "ok to ignore"? [JACKSON-77]
-        AnnotationIntrospector intr = ctxt.getAnnotationIntrospector();
-        boolean ignoreAny = false;
-        {
-            Boolean B = intr.findIgnoreUnknownProperties(beanDesc.getClassInfo());
-            if (B != null) {
-                ignoreAny = B.booleanValue();
-                builder.setIgnoreUnknownProperties(ignoreAny);
+        // 01-May-2016, tatu: Which base type to use here gets tricky, since
+        //   it may often make most sense to use general type for overrides,
+        //   but what we have here may be more specific impl type. But for now
+        //   just use it as is.
+        JsonIgnoreProperties.Value ignorals = ctxt.getConfig()
+                .getDefaultPropertyIgnorals(beanDesc.getBeanClass(),
+                        beanDesc.getClassInfo());
+        Set<String> ignored;
+
+        if (ignorals != null) {
+            boolean ignoreAny = ignorals.getIgnoreUnknown();
+            builder.setIgnoreUnknownProperties(ignoreAny);
+            // Or explicit/implicit definitions?
+            ignored = ignorals.getIgnored();
+            for (String propName : ignored) {
+                builder.addIgnorable(propName);
             }
+        } else {
+            ignored = Collections.emptySet();
         }
-        // Or explicit/implicit definitions?
-        Set<String> ignored = ArrayBuilders.arrayToSet(intr.findPropertiesToIgnore(beanDesc.getClassInfo(), false));        
-        for (String propName : ignored) {
-            builder.addIgnorable(propName);
-        }
+
         // Also, do we have a fallback "any" setter?
-        AnnotatedMethod anySetter = beanDesc.findAnySetter();
-        if (anySetter != null) {
-            builder.setAnySetter(constructAnySetter(ctxt, beanDesc, anySetter));
+        AnnotatedMethod anySetterMethod = beanDesc.findAnySetter();
+        AnnotatedMember anySetterField = null;
+        if (anySetterMethod != null) {
+            builder.setAnySetter(constructAnySetter(ctxt, beanDesc, anySetterMethod));
+        }
+        else {
+        	anySetterField = beanDesc.findAnySetterField();
+        	if(anySetterField != null) {
+        		builder.setAnySetter(constructAnySetter(ctxt, beanDesc, anySetterField));
+        	}
         }
         // NOTE: we do NOT add @JsonIgnore'd properties into blocked ones if there's any-setter
         // Implicit ones via @JsonIgnore and equivalent?
-        if (anySetter == null) {
+        if (anySetterMethod == null && anySetterField == null) {
             Collection<String> ignored2 = beanDesc.getIgnoredPropertyNames();
             if (ignored2 != null) {
                 for (String propName : ignored2) {
@@ -522,9 +532,9 @@ public class BeanDeserializerFactory
             // 25-Sep-2014, tatu: No point in finding constructor parameters for abstract types
             //   (since they are never used anyway)
             if (isConcrete && propDef.hasConstructorParameter()) {
-                /* [JACKSON-700] If property is passed via constructor parameter, we must
-                 *   handle things in special way. Not sure what is the most optimal way...
-                 *   for now, let's just call a (new) method in builder, which does nothing.
+                /* If property is passed via constructor parameter, we must
+                 * handle things in special way. Not sure what is the most optimal way...
+                 * for now, let's just call a (new) method in builder, which does nothing.
                  */
                 // but let's call a method just to allow custom builders to be aware...
                 final String name = propDef.getName();
@@ -538,8 +548,9 @@ public class BeanDeserializerFactory
                     }
                 }
                 if (cprop == null) {
-                    throw ctxt.mappingException("Could not find creator property with name '%s' (in class %s)",
+                    ctxt.reportMappingException("Could not find creator property with name '%s' (in class %s)",
                             name, beanDesc.getBeanClass().getName());
+                    continue;
                 }
                 if (prop != null) {
                     cprop.setFallbackSetter(prop);
@@ -593,9 +604,9 @@ public class BeanDeserializerFactory
                     rawPropertyType = property.getField().getRawType();
                 }
 
-                // [JACKSON-429] Some types are declared as ignorable as well
+                // Some types are declared as ignorable as well
                 if ((rawPropertyType != null)
-                        && (isIgnorableType(ctxt.getConfig(), beanDesc, rawPropertyType, ignoredTypes))) {
+                        && isIgnorableType(ctxt.getConfig(), beanDesc, rawPropertyType, ignoredTypes)) {
                     // important: make ignorable, to avoid errors if value is actually seen
                     builder.addIgnorable(name);
                     continue;
@@ -628,8 +639,8 @@ public class BeanDeserializerFactory
                 }
                 SimpleBeanPropertyDefinition propDef = SimpleBeanPropertyDefinition.construct(
                 		ctxt.getConfig(), m);
-                builder.addBackReferenceProperty(name, constructSettableProperty(
-                        ctxt, beanDesc, propDef, type));
+                builder.addBackReferenceProperty(name, constructSettableProperty(ctxt,
+                        beanDesc, propDef, type));
             }
         }
     }
@@ -644,13 +655,8 @@ public class BeanDeserializerFactory
     {
         Map<Object, AnnotatedMember> raw = beanDesc.findInjectables();
         if (raw != null) {
-            boolean fixAccess = ctxt.canOverrideAccessModifiers();
-            boolean forceAccess = fixAccess && ctxt.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS);
             for (Map.Entry<Object, AnnotatedMember> entry : raw.entrySet()) {
                 AnnotatedMember m = entry.getValue();
-                if (fixAccess) {
-                    m.fixAccess(forceAccess); // to ensure we can call it
-                }
                 builder.addInjectable(PropertyName.construct(m.getName()),
                         m.getType(),
                         beanDesc.getClassAnnotations(), m, entry.getKey());
@@ -662,37 +668,41 @@ public class BeanDeserializerFactory
      * Method called to construct fallback {@link SettableAnyProperty}
      * for handling unknown bean properties, given a method that
      * has been designated as such setter.
+     * 
+     * @param mutator Either 2-argument method (setter, with key and value), or Field
+     *     that contains Map; either way accessor used for passing "any values"
      */
+    @SuppressWarnings("unchecked")
     protected SettableAnyProperty constructAnySetter(DeserializationContext ctxt,
-            BeanDescription beanDesc, AnnotatedMethod setter)
+            BeanDescription beanDesc, AnnotatedMember mutator)
         throws JsonMappingException
     {
-        if (ctxt.canOverrideAccessModifiers()) {
-            setter.fixAccess(ctxt.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS)); // to ensure we can call it
+        //find the java type based on the annotated setter method or setter field 
+        JavaType type = null;
+        if (mutator instanceof AnnotatedMethod) {
+            // we know it's a 2-arg method, second arg is the value
+            type = ((AnnotatedMethod) mutator).getParameterType(1);
+        } else if (mutator instanceof AnnotatedField) {
+            // get the type from the content type of the map object
+            type = ((AnnotatedField) mutator).getType().getContentType();
         }
-        // we know it's a 2-arg method, second arg is the value
-        JavaType type = setter.getParameterType(1);
-        BeanProperty.Std property = new BeanProperty.Std(PropertyName.construct(setter.getName()),
-                type, null, beanDesc.getClassAnnotations(), setter,
+        // First: various annotations on type itself, as well as type-overrides
+        // on accessor need to be resolved
+        type = resolveMemberAndTypeAnnotations(ctxt, mutator, type);
+        BeanProperty.Std prop = new BeanProperty.Std(PropertyName.construct(mutator.getName()),
+                type, null, beanDesc.getClassAnnotations(), mutator,
                 PropertyMetadata.STD_OPTIONAL);
-        type = resolveType(ctxt, beanDesc, type, setter);
-
-        /* AnySetter can be annotated with @JsonDeserialize (etc) just like a
-         * regular setter... so let's see if those are used.
-         * Returns null if no annotations, in which case binding will
-         * be done at a later point.
-         */
-        JsonDeserializer<Object> deser = findDeserializerFromAnnotation(ctxt, setter);
-        /* Otherwise, method may specify more specific (sub-)class for
-         * value (no need to check if explicit deser was specified):
-         */
-        type = modifyTypeByAnnotation(ctxt, setter, type);
+        // and then possible direct deserializer override on accessor
+        JsonDeserializer<Object> deser = findDeserializerFromAnnotation(ctxt, mutator);
         if (deser == null) {
             deser = type.getValueHandler();
         }
+        if (deser != null) {
+            // As per [databind#462] need to ensure we contextualize deserializer before passing it on
+            deser = (JsonDeserializer<Object>) ctxt.handlePrimaryContextualization(deser, prop, type);
+        }
         TypeDeserializer typeDeser = type.getTypeHandler();
-        return new SettableAnyProperty(property, setter, type,
-                deser, typeDeser);
+        return new SettableAnyProperty(prop, mutator, type, deser, typeDeser);
     }
 
     /**
@@ -709,43 +719,39 @@ public class BeanDeserializerFactory
     {
         // need to ensure method is callable (for non-public)
         AnnotatedMember mutator = propDef.getNonConstructorMutator();
-        if (ctxt.canOverrideAccessModifiers()) {
-            mutator.fixAccess(ctxt.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
+        // 08-Sep-2016, tatu: issues like [databind#1342] suggest something fishy
+        //   going on; add sanity checks to try to pin down actual problem...
+        //   Possibly passing creator parameter?
+        if (mutator == null) {
+            ctxt.reportBadPropertyDefinition(beanDesc, propDef, "No non-constructor mutator available");
         }
-        // note: this works since we know there's exactly one argument for methods
-        BeanProperty.Std property = new BeanProperty.Std(propDef.getFullName(),
-                propType0, propDef.getWrapperName(),
-                beanDesc.getClassAnnotations(), mutator, propDef.getMetadata());
-        JavaType type = resolveType(ctxt, beanDesc, propType0, mutator);
-        // did type change?
-        if (type != propType0) {
-            property = property.withType(type);
-        }
-
-        /* First: does the Method specify the deserializer to use?
-         * If so, let's use it.
-         */
-        JsonDeserializer<Object> propDeser = findDeserializerFromAnnotation(ctxt, mutator);
-        type = modifyTypeByAnnotation(ctxt, mutator, type);
+        JavaType type = resolveMemberAndTypeAnnotations(ctxt, mutator, propType0);
+        // Does the Method specify the deserializer to use? If so, let's use it.
         TypeDeserializer typeDeser = type.getTypeHandler();
         SettableBeanProperty prop;
         if (mutator instanceof AnnotatedMethod) {
             prop = new MethodProperty(propDef, type, typeDeser,
-                beanDesc.getClassAnnotations(), (AnnotatedMethod) mutator);
+                    beanDesc.getClassAnnotations(), (AnnotatedMethod) mutator);
         } else {
+            // 08-Sep-2016, tatu: wonder if we should verify it is `AnnotatedField` to be safe?
             prop = new FieldProperty(propDef, type, typeDeser,
                     beanDesc.getClassAnnotations(), (AnnotatedField) mutator);
         }
-        if (propDeser != null) {
-            prop = prop.withValueDeserializer(propDeser);
+        JsonDeserializer<?> deser = findDeserializerFromAnnotation(ctxt, mutator);
+        if (deser == null) {
+            deser = type.getValueHandler();
         }
-        // [JACKSON-235]: need to retain name of managed forward references:
+        if (deser != null) {
+            deser = ctxt.handlePrimaryContextualization(deser, prop, type);
+            prop = prop.withValueDeserializer(deser);
+        }
+        // need to retain name of managed forward references:
         AnnotationIntrospector.ReferenceProperty ref = propDef.findReferenceType();
         if (ref != null && ref.isManagedReference()) {
             prop.setManagedReferenceName(ref.getName());
         }
         ObjectIdInfo objectIdInfo = propDef.findObjectIdInfo();
-        if(objectIdInfo != null){
+        if (objectIdInfo != null){
             prop.setObjectIdInfo(objectIdInfo);
         }
         return prop;
@@ -760,21 +766,17 @@ public class BeanDeserializerFactory
         throws JsonMappingException
     {
         final AnnotatedMethod getter = propDef.getGetter();
-        // need to ensure it is callable now:
-        if (ctxt.canOverrideAccessModifiers()) {
-            getter.fixAccess(ctxt.isEnabled(MapperFeature.OVERRIDE_PUBLIC_ACCESS_MODIFIERS));
-        }
-        JavaType type = getter.getType();
-        // First: does the Method specify the deserializer to use? If so, let's use it.
-        JsonDeserializer<Object> propDeser = findDeserializerFromAnnotation(ctxt, getter);
-        type = modifyTypeByAnnotation(ctxt, getter, type);
-        // As per [Issue#501], need full resolution:
-        type = resolveType(ctxt, beanDesc, type, getter);
+        JavaType type = resolveMemberAndTypeAnnotations(ctxt, getter, getter.getType());
         TypeDeserializer typeDeser = type.getTypeHandler();
         SettableBeanProperty prop = new SetterlessProperty(propDef, type, typeDeser,
                 beanDesc.getClassAnnotations(), getter);
-        if (propDeser != null) {
-            prop = prop.withValueDeserializer(propDeser);
+        JsonDeserializer<?> deser = findDeserializerFromAnnotation(ctxt, getter);
+        if (deser == null) {
+            deser = type.getValueHandler();
+        }
+        if (deser != null) {
+            deser = ctxt.handlePrimaryContextualization(deser, prop, type);
+            prop = prop.withValueDeserializer(deser);
         }
         return prop;
     }
@@ -803,7 +805,7 @@ public class BeanDeserializerFactory
             throw new IllegalArgumentException("Can not deserialize Proxy class "+type.getName()+" as a Bean");
         }
         /* also: can't deserialize some local classes: static are ok; in-method not;
-         * and with [JACKSON-594], other non-static inner classes are ok
+         * other non-static inner classes are ok
          */
         typeStr = ClassUtil.isLocalType(type, true);
         if (typeStr != null) {
@@ -823,9 +825,20 @@ public class BeanDeserializerFactory
         if (status != null) {
             return status.booleanValue();
         }
-        BeanDescription desc = config.introspectClassAnnotations(type);
-        status = config.getAnnotationIntrospector().isIgnorableType(desc.getClassInfo());
-        // We default to 'false', i.e. not ignorable
-        return (status == null) ? false : status.booleanValue(); 
+        // 21-Apr-2016, tatu: For 2.8, can specify config overrides
+        ConfigOverride override = config.findConfigOverride(type);
+        if (override != null) {
+            status = override.getIsIgnoredType();
+        }
+        if (status == null) {
+            BeanDescription desc = config.introspectClassAnnotations(type);
+            status = config.getAnnotationIntrospector().isIgnorableType(desc.getClassInfo());
+            // We default to 'false', i.e. not ignorable
+            if (status == null) {
+                status = Boolean.FALSE;
+            }
+        }
+        ignoredTypes.put(type, status);
+        return status.booleanValue();
     }
 }
