@@ -3,6 +3,7 @@ package com.fasterxml.jackson.databind.deser.std;
 import java.io.IOException;
 import java.util.*;
 
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
@@ -83,7 +84,7 @@ public class MapDeserializer
 
     // // Any properties to ignore if seen?
     
-    protected HashSet<String> _ignorableProperties;
+    protected Set<String> _ignorableProperties;
 
     /*
     /**********************************************************
@@ -131,7 +132,7 @@ public class MapDeserializer
     protected MapDeserializer(MapDeserializer src,
             KeyDeserializer keyDeser, JsonDeserializer<Object> valueDeser,
             TypeDeserializer valueTypeDeser,
-            HashSet<String> ignorable)
+            Set<String> ignorable)
     {
         super(src._mapType);
         _mapType = src._mapType;
@@ -154,7 +155,7 @@ public class MapDeserializer
     @SuppressWarnings("unchecked")
     protected MapDeserializer withResolved(KeyDeserializer keyDeser,
             TypeDeserializer valueTypeDeser, JsonDeserializer<?> valueDeser,
-            HashSet<String> ignorable)
+            Set<String> ignorable)
     {
         
         if ((_keyDeserializer == keyDeser) && (_valueDeserializer == valueDeser)
@@ -188,6 +189,11 @@ public class MapDeserializer
             null : ArrayBuilders.arrayToSet(ignorable);
     }
 
+    public void setIgnorableProperties(Set<String> ignorable) {
+        _ignorableProperties = (ignorable == null || ignorable.size() == 0) ?
+                null : ignorable;
+    }
+
     /*
     /**********************************************************
     /* Validation, post-processing (ResolvableDeserializer)
@@ -198,18 +204,28 @@ public class MapDeserializer
     public void resolve(DeserializationContext ctxt) throws JsonMappingException
     {
         // May need to resolve types for delegate- and/or property-based creators:
-        if (_valueInstantiator.canCreateUsingDelegate()) {
-            JavaType delegateType = _valueInstantiator.getDelegateType(ctxt.getConfig());
-            if (delegateType == null) {
-                throw new IllegalArgumentException("Invalid delegate-creator definition for "+_mapType
-                        +": value instantiator ("+_valueInstantiator.getClass().getName()
-                        +") returned true for 'canCreateUsingDelegate()', but null for 'getDelegateType()'");
+        if (_valueInstantiator != null) {
+            if (_valueInstantiator.canCreateUsingDelegate()) {
+                JavaType delegateType = _valueInstantiator.getDelegateType(ctxt.getConfig());
+                if (delegateType == null) {
+                    throw new IllegalArgumentException("Invalid delegate-creator definition for "+_mapType
+                            +": value instantiator ("+_valueInstantiator.getClass().getName()
+                            +") returned true for 'canCreateUsingDelegate()', but null for 'getDelegateType()'");
+                }
+                /* Theoretically should be able to get CreatorProperty for delegate
+                 * parameter to pass; but things get tricky because DelegateCreator
+                 * may contain injectable values. So, for now, let's pass nothing.
+                 */
+                _delegateDeserializer = findDeserializer(ctxt, delegateType, null);
+            } else if (_valueInstantiator.canCreateUsingArrayDelegate()) {
+                JavaType delegateType = _valueInstantiator.getArrayDelegateType(ctxt.getConfig());
+                if (delegateType == null) {
+                    throw new IllegalArgumentException("Invalid delegate-creator definition for "+_mapType
+                            +": value instantiator ("+_valueInstantiator.getClass().getName()
+                            +") returned true for 'canCreateUsingDelegate()', but null for 'getArrayDelegateType()'");
+                }
+                _delegateDeserializer = findDeserializer(ctxt, delegateType, null);
             }
-            /* Theoretically should be able to get CreatorProperty for delegate
-             * parameter to pass; but things get tricky because DelegateCreator
-             * may contain injectable values. So, for now, let's pass nothing.
-             */
-            _delegateDeserializer = findDeserializer(ctxt, delegateType, null);
         }
         if (_valueInstantiator.canCreateFromObjectWith()) {
             SettableBeanProperty[] creatorProps = _valueInstantiator.getFromObjectArguments(ctxt.getConfig());
@@ -234,8 +250,9 @@ public class MapDeserializer
                 kd = ((ContextualKeyDeserializer) kd).createContextual(ctxt, property);
             }
         }
+        
         JsonDeserializer<?> vd = _valueDeserializer;
-        // #125: May have a content converter
+        // [databind#125]: May have a content converter
         if (property != null) {
             vd = findConvertingContentDeserializer(ctxt, property, vd);
         }
@@ -249,16 +266,19 @@ public class MapDeserializer
         if (vtd != null) {
             vtd = vtd.forProperty(property);
         }
-        HashSet<String> ignored = _ignorableProperties;
+        Set<String> ignored = _ignorableProperties;
         AnnotationIntrospector intr = ctxt.getAnnotationIntrospector();
         if (intr != null && property != null) {
             AnnotatedMember member = property.getMember();
             if (member != null) {
-                String[] moreToIgnore = intr.findPropertiesToIgnore(member, false);
-                if (moreToIgnore != null) {
-                    ignored = (ignored == null) ? new HashSet<String>() : new HashSet<String>(ignored);
-                    for (String str : moreToIgnore) {
-                        ignored.add(str);
+                JsonIgnoreProperties.Value ignorals = intr.findPropertyIgnorals(member);
+                if (ignorals != null) {
+                    Set<String> ignoresToAdd = ignorals.findIgnoredForDeserialization();
+                    if (!ignoresToAdd.isEmpty()) {
+                        ignored = (ignored == null) ? new HashSet<String>() : new HashSet<String>(ignored);
+                        for (String str : ignoresToAdd) {
+                            ignored.add(str);
+                        }
                     }
                 }
             }
@@ -324,12 +344,13 @@ public class MapDeserializer
                     _delegateDeserializer.deserialize(p, ctxt));
         }
         if (!_hasDefaultCreator) {
-            throw ctxt.instantiationException(getMapClass(), "No default constructor found");
+            return (Map<Object,Object> ) ctxt.handleMissingInstantiator(getMapClass(), p,
+                    "no default constructor found");
         }
         // Ok: must point to START_OBJECT, FIELD_NAME or END_OBJECT
         JsonToken t = p.getCurrentToken();
         if (t != JsonToken.START_OBJECT && t != JsonToken.FIELD_NAME && t != JsonToken.END_OBJECT) {
-            // [JACKSON-620] (empty) String may be ok however:
+            // (empty) String may be ok however:
             if (t == JsonToken.VALUE_STRING) {
                 return (Map<Object,Object>) _valueInstantiator.createFromString(ctxt, p.getText());
             }
@@ -338,13 +359,14 @@ public class MapDeserializer
         }
         final Map<Object,Object> result = (Map<Object,Object>) _valueInstantiator.createUsingDefault(ctxt);
         if (_standardStringKey) {
-            _readAndBindStringMap(p, ctxt, result);
+            _readAndBindStringKeyMap(p, ctxt, result);
             return result;
         }
         _readAndBind(p, ctxt, result);
         return result;
     }
 
+    @SuppressWarnings("unchecked")
     @Override
     public Map<Object,Object> deserialize(JsonParser p, DeserializationContext ctxt,
             Map<Object,Object> result)
@@ -356,10 +378,10 @@ public class MapDeserializer
         // Ok: must point to START_OBJECT or FIELD_NAME
         JsonToken t = p.getCurrentToken();
         if (t != JsonToken.START_OBJECT && t != JsonToken.FIELD_NAME) {
-            throw ctxt.mappingException(getMapClass());
+            return (Map<Object,Object>) ctxt.handleUnexpectedToken(getMapClass(), p);
         }
         if (_standardStringKey) {
-            _readAndBindStringMap(p, ctxt, result);
+            _readAndBindStringKeyMap(p, ctxt, result);
             return result;
         }
         _readAndBind(p, ctxt, result);
@@ -398,7 +420,7 @@ public class MapDeserializer
         final KeyDeserializer keyDes = _keyDeserializer;
         final JsonDeserializer<Object> valueDes = _valueDeserializer;
         final TypeDeserializer typeDeser = _valueTypeDeserializer;
-
+        
         MapReferringAccumulator referringAccumulator = null;
         boolean useObjectId = valueDes.getObjectIdReader() != null;
         if (useObjectId) {
@@ -414,7 +436,7 @@ public class MapDeserializer
                 return;
             }
             if (t != JsonToken.FIELD_NAME) {
-                throw ctxt.mappingException(_mapType.getRawClass(), p.getCurrentToken());
+                ctxt.reportWrongTokenException(p, JsonToken.FIELD_NAME, null);
             }
             keyStr = p.getCurrentName();
         }
@@ -455,7 +477,7 @@ public class MapDeserializer
      * {@link java.lang.String}s, and there is no custom deserialized
      * specified.
      */
-    protected final void _readAndBindStringMap(JsonParser p, DeserializationContext ctxt,
+    protected final void _readAndBindStringKeyMap(JsonParser p, DeserializationContext ctxt,
             Map<Object,Object> result) throws IOException
     {
         final JsonDeserializer<Object> valueDes = _valueDeserializer;
@@ -465,7 +487,7 @@ public class MapDeserializer
         if (useObjectId) {
             referringAccumulator = new MapReferringAccumulator(_mapType.getContentType().getRawClass(), result);
         }
-
+        
         String key;
         if (p.isExpectedStartObjectToken()) {
             key = p.nextFieldName();
@@ -475,7 +497,7 @@ public class MapDeserializer
                 return;
             }
             if (t != JsonToken.FIELD_NAME) {
-                throw ctxt.mappingException(_mapType.getRawClass(), p.getCurrentToken());
+                ctxt.reportWrongTokenException(p, JsonToken.FIELD_NAME, null);
             }
             key = p.getCurrentName();
         }
@@ -655,7 +677,7 @@ public class MapDeserializer
      * The resolved object associated with {@link #key} comes before the values in
      * {@link #next}.
      */
-    final static class MapReferring extends Referring {
+    static class MapReferring extends Referring {
         private final MapReferringAccumulator _parent;
 
         public final Map<Object, Object> next = new LinkedHashMap<Object, Object>();
