@@ -6,6 +6,7 @@ import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.annotation.JsonInclude;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonPOJOBuilder;
 import com.fasterxml.jackson.databind.cfg.HandlerInstantiator;
@@ -22,9 +23,6 @@ import com.fasterxml.jackson.databind.util.Converter;
  * use cases for that, nor is such usage tested or supported.
  * Separation from API is mostly to isolate some implementation details
  * here and keep API simple.
- *<p>
- * Note that since 2.6 this class has been a thin shell around
- * {@link POJOPropertiesCollector}, which handles most of actual work.
  */
 public class BasicBeanDescription extends BeanDescription
 {
@@ -49,12 +47,6 @@ public class BasicBeanDescription extends BeanDescription
      * Information collected about the class introspected.
      */
     final protected AnnotatedClass _classInfo;
-    
-    /**
-     * We may need type bindings for the bean type. If so, we'll
-     * construct it lazily
-     */
-    protected TypeBindings _bindings;
 
     /*
     /**********************************************************
@@ -71,7 +63,7 @@ public class BasicBeanDescription extends BeanDescription
      * Details of Object Id to include, if any
      */
     protected ObjectIdInfo _objectIdInfo;
-    
+
     /*
     /**********************************************************
     /* Life-cycle
@@ -84,7 +76,12 @@ public class BasicBeanDescription extends BeanDescription
         super(type);
         _propCollector = coll;
         _config = coll.getConfig();
-        _annotationIntrospector = (_config == null) ? null : _config.getAnnotationIntrospector();
+        // NOTE: null config only for some pre-constructed types
+        if (_config == null) {
+            _annotationIntrospector = null;
+        } else {
+            _annotationIntrospector = _config.getAnnotationIntrospector();
+        }
         _classInfo = classDef;
     }
 
@@ -98,7 +95,12 @@ public class BasicBeanDescription extends BeanDescription
         super(type);
         _propCollector = null;
         _config = config;
-        _annotationIntrospector = (_config == null) ? null : _config.getAnnotationIntrospector();
+        // NOTE: null config only for some pre-constructed types
+        if (_config == null) {
+            _annotationIntrospector = null;
+        } else {
+            _annotationIntrospector = _config.getAnnotationIntrospector();
+        }
         _classInfo = classDef;
         _properties = props;
     }
@@ -250,6 +252,7 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     @Override
+    @Deprecated // since 2.8
     public JavaType resolveType(java.lang.reflect.Type jdkType) {
         if (jdkType == null) {
             return null;
@@ -339,10 +342,24 @@ public class BasicBeanDescription extends BeanDescription
     @Override
     public JsonFormat.Value findExpectedFormat(JsonFormat.Value defValue)
     {
+        // 15-Apr-2016, tatu: Let's check both per-type defaults and annotations; per-type
+        //   defaults having higher precedence, so start with that
         if (_annotationIntrospector != null) {
             JsonFormat.Value v = _annotationIntrospector.findFormat(_classInfo);
             if (v != null) {
-                return v;
+                if (defValue == null) {
+                    defValue = v;
+                } else {
+                    defValue = defValue.withOverrides(v);
+                }
+            }
+        }
+        JsonFormat.Value v = _config.getDefaultPropertyFormat(_classInfo.getRawType());
+        if (v != null) {
+            if (defValue == null) {
+                defValue = v;
+            } else {
+                defValue = defValue.withOverrides(v);
             }
         }
         return defValue;
@@ -374,7 +391,7 @@ public class BasicBeanDescription extends BeanDescription
         if (_annotationIntrospector != null) {
             JsonInclude.Value incl = _annotationIntrospector.findPropertyInclusion(_classInfo);
             if (incl != null) {
-                return defValue.withOverrides(incl);
+                return (defValue == null) ? incl : defValue.withOverrides(incl);
             }
         }
         return defValue;
@@ -403,6 +420,23 @@ public class BasicBeanDescription extends BeanDescription
         return anyGetter;
     }
 
+    @Override
+    public AnnotatedMember findAnySetterField() throws IllegalArgumentException {
+        AnnotatedMember anySetter = (_propCollector == null) ? null : _propCollector.getAnySetterField();
+		if (anySetter != null) {
+			/*
+			 * For now let's require a Map; in future can add support for other
+			 * types like perhaps Iterable<Map.Entry>?
+			 */
+			Class<?> type = anySetter.getRawType();
+			if (!Map.class.isAssignableFrom(type)) {
+				throw new IllegalArgumentException("Invalid 'any-setter' annotation on field " + anySetter.getName()
+				        + "(): type is not instance of java.util.Map");
+			}
+		}
+		return anySetter;
+	}
+
     @Override
     public Map<String,AnnotatedMember> findBackReferenceProperties()
     {
@@ -487,7 +521,8 @@ public class BasicBeanDescription extends BeanDescription
     {
         // So, of all single-arg static methods:
         for (AnnotatedMethod am : _classInfo.getStaticMethods()) {
-            if (isFactoryMethod(am)) {
+            // 24-Oct-2016, tatu: Better ensure it only takes 1 arg, no matter what
+            if (isFactoryMethod(am) && am.getParameterCount() == 1) {
                 // And must take one of expected arg types (or supertype)
                 Class<?> actualArgType = am.getRawParameterType(0);
                 for (Class<?> expArgType : expArgTypes) {
@@ -504,14 +539,12 @@ public class BasicBeanDescription extends BeanDescription
     protected boolean isFactoryMethod(AnnotatedMethod am)
     {
         /* First: return type must be compatible with the introspected class
-         * (i.e. allowed to be sub-class, although usually is the same
-         * class)
+         * (i.e. allowed to be sub-class, although usually is the same class)
          */
         Class<?> rt = am.getRawReturnType();
         if (!getBeanClass().isAssignableFrom(rt)) {
             return false;
         }
-
         /* Also: must be a recognized factory method, meaning:
          * (a) marked with @JsonCreator annotation, or
          * (b) "valueOf" (at this point, need not be public)
@@ -520,12 +553,15 @@ public class BasicBeanDescription extends BeanDescription
             return true;
         }
         final String name = am.getName();
+        // 24-Oct-2016, tatu: As per [databind#1429] must ensure takes exactly one arg
         if ("valueOf".equals(name)) {
-            return true;
+            if (am.getParameterCount() == 1) {
+                return true;
+            }
         }
-        // [Issue#208] Also accept "fromString()", if takes String or CharSequence
+        // [databind#208] Also accept "fromString()", if takes String or CharSequence
         if ("fromString".equals(name)) {
-            if (1 == am.getParameterCount()) {
+            if (am.getParameterCount() == 1) {
                 Class<?> cls = am.getRawParameterType(0);
                 if (cls == String.class || CharSequence.class.isAssignableFrom(cls)) {
                     return true;
@@ -536,50 +572,9 @@ public class BasicBeanDescription extends BeanDescription
     }
 
     /**
-     * @deprecated Since 2.4, use <code>findCreatorParameterNames()</code> instead.
+     * @deprecated since 2.8
      */
-    @Deprecated
-    public List<String> findCreatorPropertyNames()
-    {
-        List<PropertyName> params = findCreatorParameterNames();
-        if (params.isEmpty()) {
-    		    return Collections.emptyList();
-        }
-        List<String> result = new ArrayList<String>(params.size());
-        for (PropertyName name : params) {
-    		    result.add(name.getSimpleName());
-        }
-        return result;
-    }
-    
-    /**
-     * @deprecated Since 2.5, does not seem to be used at all.
-     */
-    @Deprecated
-    public List<PropertyName> findCreatorParameterNames()
-    {
-        for (int i = 0; i < 2; ++i) {
-            List<? extends AnnotatedWithParams> l = (i == 0)
-                ? getConstructors() : getFactoryMethods();
-            for (AnnotatedWithParams creator : l) {
-                int argCount = creator.getParameterCount();
-                if (argCount < 1) continue;
-                PropertyName name = _findCreatorPropertyName(creator.getParameter(0));
-                if (name == null || name.isEmpty()) {
-                    continue;
-                }
-                List<PropertyName> names = new ArrayList<PropertyName>();
-                names.add(name);
-                for (int p = 1; p < argCount; ++p) {
-                    name = _findCreatorPropertyName(creator.getParameter(p));
-                    names.add(name);
-                }
-                return names;
-            }
-        }
-        return Collections.emptyList();
-    }
-
+    @Deprecated // since 2.8, not used at least since 2.7
     protected PropertyName _findCreatorPropertyName(AnnotatedParameter param)
     {
         PropertyName name = _annotationIntrospector.findNameForDeserialization(param);
